from os import environ
from unittest import TestCase

from memcacheify import memcacheify


class Memcacheify(TestCase):

    def test_uses_local_memory_backend_if_no_memcache_addon_is_available(self):
        self.assertEqual(memcacheify(), {
            'default': {'BACKEND': 'django.core.cache.backends.locmem.LocMemCache'}
        })

    def tests_uses_local_memory_backend_if_one_of_the_memcache_env_vars_is_missing(self):
        environ['MEMCACHE_PASSWORD'] = 'GCnQ9DhfEJqNDlo1'
        environ['MEMCACHE_SERVERS'] = 'mc3.ec2.northscale.net'
        self.assertEqual(memcacheify(), {
            'default': {'BACKEND': 'django.core.cache.backends.locmem.LocMemCache'}
        })
        del environ['MEMCACHE_PASSWORD']
        del environ['MEMCACHE_SERVERS']

    def test_sets_proper_backend_when_memcache_addon_is_available(self):
        environ['MEMCACHE_PASSWORD'] = 'GCnQ9DhfEJqNDlo1'
        environ['MEMCACHE_SERVERS'] = 'mc3.ec2.northscale.net'
        environ['MEMCACHE_USERNAME'] = 'appxxxxx%40heroku.com'
        self.assertEqual(memcacheify()['default']['BACKEND'],
                         'django_pylibmc.memcached.PyLibMCCache')
        del environ['MEMCACHE_PASSWORD']
        del environ['MEMCACHE_SERVERS']
        del environ['MEMCACHE_USERNAME']

    def test_uses_local_memory_backend_if_no_memcachier_addon_is_available(self):
        environ['MEMCACHIER_PASSWORD'] = 'xxx'
        environ['MEMCACHIER_SERVERS'] = 'mc1.ec2.memcachier.com'
        self.assertEqual(memcacheify(), {
            'default': {'BACKEND': 'django.core.cache.backends.locmem.LocMemCache'}
        })
        del environ['MEMCACHIER_PASSWORD']
        del environ['MEMCACHIER_SERVERS']

    def test_sets_proper_backend_when_memcachier_addon_is_available(self):
        environ['MEMCACHIER_PASSWORD'] = 'xxx'
        environ['MEMCACHIER_SERVERS'] = 'mc1.ec2.memcachier.com'
        environ['MEMCACHIER_USERNAME'] = 'xxx'

        caches = memcacheify()
        self.assertEqual(caches['default']['BACKEND'], 'django_pylibmc.memcached.PyLibMCCache')
        self.assertEqual(environ['MEMCACHE_SERVERS'], environ['MEMCACHIER_SERVERS'])
        self.assertEqual(environ['MEMCACHE_USERNAME'], environ['MEMCACHIER_USERNAME'])
        self.assertEqual(environ['MEMCACHE_PASSWORD'], environ['MEMCACHIER_PASSWORD'])

        del environ['MEMCACHIER_PASSWORD']
        del environ['MEMCACHIER_SERVERS']
        del environ['MEMCACHIER_USERNAME']
        del environ['MEMCACHE_PASSWORD']
        del environ['MEMCACHE_SERVERS']
        del environ['MEMCACHE_USERNAME']

    def test_sets_proper_backend_when_memcachedcloud_addon_is_available(self):
        environ['MEMCACHEDCLOUD_PASSWORD'] = 'xyz'
        environ['MEMCACHEDCLOUD_SERVERS'] = 'zzzz'
        environ['MEMCACHEDCLOUD_USERNAME'] = 'xyzzy'

        caches = memcacheify()
        self.assertEqual(caches['default']['BACKEND'], 'django_pylibmc.memcached.PyLibMCCache')
        self.assertEqual(environ['MEMCACHE_SERVERS'], environ['MEMCACHEDCLOUD_SERVERS'])
        self.assertEqual(environ['MEMCACHE_USERNAME'], environ['MEMCACHEDCLOUD_USERNAME'])
        self.assertEqual(environ['MEMCACHE_PASSWORD'], environ['MEMCACHEDCLOUD_PASSWORD'])

        del environ['MEMCACHEDCLOUD_PASSWORD']
        del environ['MEMCACHEDCLOUD_SERVERS']
        del environ['MEMCACHEDCLOUD_USERNAME']
        del environ['MEMCACHE_PASSWORD']
        del environ['MEMCACHE_SERVERS']
        del environ['MEMCACHE_USERNAME']



from os.path import abspath, dirname, join, normpath

from setuptools import setup


setup(

    # Basic package information:
    name='django-heroku-memcacheify',
    version='1.0.1',
    py_modules=('memcacheify',),

    # Packaging options:
    zip_safe=False,
    include_package_data=True,

    # Package dependencies:
    install_requires=['django-pylibmc>=0.6.1'],

    # Metadata for PyPI:
    author='Randall Degges',
    author_email='r@rdegges.com',
    license='UNLICENSE',
    url='https://github.com/rdegges/django-heroku-memcacheify',
    keywords='django heroku cloud cache memcache memcached awesome epic',
    classifiers=[
        'Framework :: Django',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
    ],
    description='Automatic Django memcached configuration on Heroku.',
    long_description=open(normpath(join(dirname(abspath(__file__)),
                                        'README.md'))).read(),
    long_description_content_type='text/markdown'

)



from os import environ


# Memcache addon environment variables.
# See: https://addons.heroku.com/memcache
MEMCACHE_ENV_VARS = (
    'MEMCACHE_PASSWORD',
    'MEMCACHE_SERVERS',
    'MEMCACHE_USERNAME',
)


# MemCachier addon environment variables.
# See: https://addons.heroku.com/memcachier
MEMCACHIER_ENV_VARS = (
    'MEMCACHIER_PASSWORD',
    'MEMCACHIER_SERVERS',
    'MEMCACHIER_USERNAME',
)


# Memcached Cloud
# See: https://addons.heroku.com/memcachedcloud
MEMCACHEDCLOUD_ENV_VARS = (
    'MEMCACHEDCLOUD_PASSWORD',
    'MEMCACHEDCLOUD_SERVERS',
    'MEMCACHEDCLOUD_USERNAME',
)

# Memcache defaults
# Both Memcachier and MemcacheCloud share these
CACHE_DEFAULTS = {
    'BACKEND': 'django_pylibmc.memcached.PyLibMCCache',
    'BINARY': True,
    'OPTIONS': {
        'ketama': True,
        'tcp_nodelay': True,
    },
}


def memcacheify(timeout=500):
    """Return a fully configured Django ``CACHES`` setting. We do this by
    analyzing all environment variables on Heorku, scanning for an available
    memcache addon, and then building the settings dict properly.

    If no memcache servers can be found, we'll revert to building a local
    memory cache.

    Returns a fully configured caches dict.
    """
    caches = {}

    if all((environ.get(e, '') for e in MEMCACHE_ENV_VARS)):
        caches['default'] = CACHE_DEFAULTS
        caches['default'].update({
            'LOCATION': 'localhost:11211',
            'TIMEOUT': timeout,
        })
    elif all((environ.get(e, '') for e in MEMCACHIER_ENV_VARS)):
        servers = environ.get('MEMCACHIER_SERVERS').replace(',', ';')
        environ['MEMCACHE_SERVERS'] = servers
        environ['MEMCACHE_USERNAME'] = environ.get('MEMCACHIER_USERNAME')
        environ['MEMCACHE_PASSWORD'] = environ.get('MEMCACHIER_PASSWORD')
        caches['default'] = CACHE_DEFAULTS
        caches['default'].update({
            'LOCATION': servers,
            'TIMEOUT': timeout,
        })
    elif all((environ.get(e, '') for e in MEMCACHEDCLOUD_ENV_VARS)):
        servers = environ.get('MEMCACHEDCLOUD_SERVERS').replace(',', ';')
        environ['MEMCACHE_SERVERS'] = servers
        environ['MEMCACHE_USERNAME'] = environ.get('MEMCACHEDCLOUD_USERNAME')
        environ['MEMCACHE_PASSWORD'] = environ.get('MEMCACHEDCLOUD_PASSWORD')
        caches['default'] = CACHE_DEFAULTS
        caches['default'].update({
            'LOCATION': servers,
            'TIMEOUT': timeout,
        })
    elif environ.get('MEMCACHEIFY_USE_LOCAL', False):
        caches['default'] = {
            'BACKEND': 'django_pylibmc.memcached.PyLibMCCache',
        }
    else:
        caches['default'] = {
            'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
        }

    return caches
